@@ -22,8 +22,9 @@
 #include <stdio.h>
 #include <stdint.h>
 #include <string.h>
+#include <stdbool.h>
 #include <urcu/tls-compat.h>
-#include <time.h>
+#include <common/compat/time.h>
 
 #ifndef _GNU_SOURCE
 #error "lttng-tools error.h needs _GNU_SOURCE"
@@ -32,6 +33,11 @@
 #include <lttng/lttng-error.h>
 #include <common/compat/tid.h>
 
+/* Avoid conflict with Solaris <sys/regset.h> */
+#if defined(ERR) && defined(__sun__)
+#undef ERR
+#endif
+
 /* Stringify the expansion of a define */
 #define XSTR(d) STR(d)
 #define STR(s) #s
@@ -52,13 +58,79 @@ extern int lttng_opt_verbose;
 extern int lttng_opt_mi;
 
 /* Error type. */
-#define PRINT_ERR   0x1
-#define PRINT_WARN  0x2
-#define PRINT_BUG   0x3
-#define PRINT_MSG   0x4
-#define PRINT_DBG   0x10
-#define PRINT_DBG2  0x20
-#define PRINT_DBG3  0x30
+enum lttng_error_level {
+	PRINT_ERR =	0,
+	PRINT_BUG =	1,
+	PRINT_WARN =	2,
+	PRINT_MSG =	3,
+	PRINT_DBG =	4,
+	PRINT_DBG2 =	5,
+	PRINT_DBG3 =	6,
+};
+
+static inline bool __lttng_print_check_opt(enum lttng_error_level type)
+{
+	/* lttng_opt_mi and lttng_opt_quiet. */
+	switch (type) {
+	case PRINT_DBG3:
+	case PRINT_DBG2:
+	case PRINT_DBG:
+	case PRINT_MSG:
+		if (lttng_opt_mi) {
+			return false;
+		}
+		/* Fall-through. */
+	case PRINT_WARN:
+	case PRINT_BUG:
+	case PRINT_ERR:
+		if (lttng_opt_quiet) {
+			return false;
+		}
+	}
+
+	/* lttng_opt_verbose */
+	switch (type) {
+	case PRINT_DBG3:
+		if (lttng_opt_verbose < 3) {
+			return false;
+		}
+		break;
+	case PRINT_DBG2:
+		if (lttng_opt_verbose < 2) {
+			return false;
+		}
+		break;
+	case PRINT_DBG:
+		if (lttng_opt_verbose < 1) {
+			return false;
+		}
+		break;
+	case PRINT_MSG:
+	case PRINT_WARN:
+	case PRINT_BUG:
+	case PRINT_ERR:
+		break;
+	}
+
+	return true;
+}
+
+void lttng_abort_on_error(void);
+
+static inline void __lttng_print_check_abort(enum lttng_error_level type)
+{
+	switch (type) {
+	case PRINT_DBG3:
+	case PRINT_DBG2:
+	case PRINT_DBG:
+	case PRINT_MSG:
+	case PRINT_WARN:
+		break;
+	case PRINT_BUG:
+	case PRINT_ERR:
+		lttng_abort_on_error();
+	}
+}
 
 /*
  * Macro for printing message depending on command line option and verbosity.
@@ -68,24 +140,13 @@ extern int lttng_opt_mi;
  * want any nested msg to show up when printing mi to stdout(if it's the case).
  * All warnings and errors should be printed to stderr as normal.
  */
-#define __lttng_print(type, fmt, args...)                           \
-	do {                                                            \
-		if (lttng_opt_quiet == 0 && lttng_opt_mi == 0 &&            \
-				type == PRINT_MSG) {                                \
-			fprintf(stdout, fmt, ## args);                          \
-		} else if (lttng_opt_quiet == 0 && lttng_opt_mi == 0 &&     \
-				(((type & PRINT_DBG) && lttng_opt_verbose == 1) ||  \
-				((type & (PRINT_DBG | PRINT_DBG2)) &&               \
-					lttng_opt_verbose == 2) ||                      \
-				((type & (PRINT_DBG | PRINT_DBG2 | PRINT_DBG3)) &&  \
-					lttng_opt_verbose == 3))) {                     \
-			fprintf(stderr, fmt, ## args);                          \
-		} else if (lttng_opt_quiet == 0 && (type & (PRINT_WARN))) { \
-			fprintf(stderr, fmt, ## args);                          \
-		} else if (type & (PRINT_ERR | PRINT_BUG)) {                \
-			fprintf(stderr, fmt, ## args);                          \
-		}                                                           \
-	} while (0);
+#define __lttng_print(type, fmt, args...)						\
+	do {										\
+		if (__lttng_print_check_opt(type)) {					\
+			fprintf((type) == PRINT_MSG ? stdout : stderr, fmt, ## args);	\
+		}									\
+		__lttng_print_check_abort(type);					\
+	} while (0)
 
 /* Three level of debug. Use -v, -vv or -vvv for the levels */
 #define _ERRMSG(msg, type, fmt, args...) __lttng_print(type, msg \
@@ -99,17 +160,45 @@ extern int lttng_opt_mi;
 #define ERR(fmt, args...) \
 	__lttng_print(PRINT_ERR, "Error: " fmt "\n", ## args)
 #define WARN(fmt, args...) \
-	__lttng_print(PRINT_ERR, "Warning: " fmt "\n", ## args)
+	__lttng_print(PRINT_WARN, "Warning: " fmt "\n", ## args)
 
 #define BUG(fmt, args...) _ERRMSG("BUG", PRINT_BUG, fmt, ## args)
 
 #define DBG(fmt, args...) _ERRMSG("DEBUG1", PRINT_DBG, fmt, ## args)
 #define DBG2(fmt, args...) _ERRMSG("DEBUG2", PRINT_DBG2, fmt, ## args)
 #define DBG3(fmt, args...) _ERRMSG("DEBUG3", PRINT_DBG3, fmt, ## args)
+#define LOG(type, fmt, args...)			\
+	do {					\
+		switch (type) {			\
+		case PRINT_ERR:			\
+			ERR(fmt, ## args);	\
+			break;			\
+		case PRINT_WARN:		\
+			WARN(fmt, ## args);	\
+			break;			\
+		case PRINT_BUG:			\
+			BUG(fmt, ## args);	\
+			break;			\
+		case PRINT_MSG:			\
+			MSG(fmt, ## args);	\
+			break;			\
+		case PRINT_DBG:			\
+			DBG(fmt, ## args);	\
+			break;			\
+		case PRINT_DBG2:		\
+			DBG2(fmt, ## args);	\
+			break;			\
+		case PRINT_DBG3:		\
+			DBG3(fmt, ## args);	\
+			break;			\
+		default:			\
+			assert(0);		\
+		}				\
+	} while(0);
 
 #define _PERROR(fmt, args...) _ERRMSG("PERROR", PRINT_ERR, fmt, ## args)
 
-#if !defined(__linux__) || ((_POSIX_C_SOURCE >= 200112L || _XOPEN_SOURCE >= 600) && !defined(_GNU_SOURCE))
+#if !defined(__GLIBC__) || ((_POSIX_C_SOURCE >= 200112L || _XOPEN_SOURCE >= 600) && !defined(_GNU_SOURCE))
 
 /*
  * Version using XSI strerror_r.
